@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-# !/usr/bin/env python
 # Copyright 2014 Objectif Libre
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -33,6 +32,7 @@ from cloudkitty import config  # noqa
 from cloudkitty import extension_manager
 from cloudkitty import messaging
 from cloudkitty import storage
+from cloudkitty import storage_state as state
 from cloudkitty import transformer
 from cloudkitty import utils as ck_utils
 
@@ -41,7 +41,6 @@ eventlet.monkey_patch()
 LOG = logging.getLogger(__name__)
 
 CONF = cfg.CONF
-CONF.import_opt('backend', 'cloudkitty.tenant_fetcher', 'tenant_fetcher')
 
 orchestrator_opts = [
     cfg.StrOpt('coordination_url',
@@ -51,8 +50,12 @@ orchestrator_opts = [
 ]
 CONF.register_opts(orchestrator_opts, group='orchestrator')
 
-FETCHERS_NAMESPACE = 'cloudkitty.tenant.fetchers'
+CONF.import_opt('backend', 'cloudkitty.fetcher', 'fetcher')
+
+FETCHERS_NAMESPACE = 'cloudkitty.fetchers'
 PROCESSORS_NAMESPACE = 'cloudkitty.rating.processors'
+COLLECTORS_NAMESPACE = 'cloudkitty.collector.backends'
+STORAGES_NAMESPACE = 'cloudkitty.storage.backends'
 
 
 class RatingEndpoint(object):
@@ -149,30 +152,37 @@ class APIWorker(BaseWorker):
 
 
 class Worker(BaseWorker):
-    def __init__(self, collector, storage, tenant_id=None):
+    def __init__(self, collector, storage, tenant_id):
         self._collector = collector
         self._storage = storage
         self._period = CONF.collect.period
         self._wait_time = CONF.collect.wait_periods * self._period
+        self._tenant_id = tenant_id
+        self._conf = ck_utils.load_conf(CONF.collect.metrics_conf)
+        self._state = state.StateManager()
 
-        super(Worker, self).__init__(tenant_id)
+        super(Worker, self).__init__(self._tenant_id)
 
-    def _collect(self, service, start_timestamp):
+    def _collect(self, metric, start_timestamp):
         next_timestamp = start_timestamp + self._period
-        raw_data = self._collector.retrieve(service,
-                                            start_timestamp,
-                                            next_timestamp,
-                                            self._tenant_id)
+
+        raw_data = self._collector.retrieve(
+            metric,
+            start_timestamp,
+            next_timestamp,
+            self._tenant_id,
+        )
+
         if raw_data:
             return [{'period': {'begin': start_timestamp,
                                 'end': next_timestamp},
                      'usage': raw_data}]
 
     def check_state(self):
-        timestamp = self._storage.get_state(self._tenant_id)
+        timestamp = self._state.get_state(self._tenant_id)
         return ck_utils.check_time_state(timestamp,
                                          self._period,
-                                         self._wait_time)
+                                         CONF.collect.wait_periods)
 
     def run(self):
         while True:
@@ -180,46 +190,64 @@ class Worker(BaseWorker):
             if not timestamp:
                 break
 
-            for service in CONF.collect.services:
+            metrics = list(self._conf['metrics'].keys())
+
+            storage_data = []
+            for metric in metrics:
                 try:
                     try:
-                        data = self._collect(service, timestamp)
+                        data = self._collect(metric, timestamp)
                     except collector.NoDataCollected:
                         raise
                     except Exception as e:
                         LOG.warning(
-                            'Error while collecting service '
-                            '%(service)s: %(error)s',
-                            {'service': service, 'error': e})
-                        raise collector.NoDataCollected('', service)
+                            '[%(scope_id)s] Error while collecting metric '
+                            '%(metric)s: %(error)s. Retrying on next '
+                            'collection cycle.',
+                            {
+                                'scope_id': self._tenant_id,
+                                'metric': metric,
+                                'error': e,
+                            },
+                        )
+                        # FIXME(peschk_l): here we just exit, and the
+                        # collection will be retried during the next collect
+                        # cycle. In the future, we should implement a retrying
+                        # system in workers
+                        return
                 except collector.NoDataCollected:
-                    begin = timestamp
-                    end = begin + self._period
-                    for processor in self._processors:
-                        processor.obj.nodata(begin, end)
-                    self._storage.nodata(begin, end, self._tenant_id)
+                    LOG.info(
+                        '[{}] No data collected for metric {} '
+                        'at timestamp {}'.format(
+                            self._tenant_id, metric, ck_utils.ts2dt(timestamp))
+                    )
                 else:
                     # Rating
                     for processor in self._processors:
                         processor.obj.process(data)
                     # Writing
-                    self._storage.append(data, self._tenant_id)
+                    if isinstance(data, list):
+                        storage_data += data
+                    else:
+                        storage_data.append(data)
 
             # We're getting a full period so we directly commit
-            self._storage.commit(self._tenant_id)
+            self._storage.push(storage_data, self._tenant_id)
+            self._state.set_state(self._tenant_id, timestamp)
 
 
 class Orchestrator(object):
     def __init__(self):
-        # Tenant fetcher
         self.fetcher = driver.DriverManager(
             FETCHERS_NAMESPACE,
-            CONF.tenant_fetcher.backend,
-            invoke_on_load=True).driver
+            CONF.fetcher.backend,
+            invoke_on_load=True,
+        ).driver
 
-        self.transformers = transformer.get_transformers()
-        self.collector = collector.get_collector(self.transformers)
-        self.storage = storage.get_storage(self.collector)
+        transformers = transformer.get_transformers()
+        self.collector = collector.get_collector(transformers)
+        self.storage = storage.get_storage()
+        self._state = state.StateManager()
 
         # RPC
         self.server = None
@@ -230,19 +258,12 @@ class Orchestrator(object):
         self.coord = coordination.get_coordinator(
             CONF.orchestrator.coordination_url,
             uuidutils.generate_uuid().encode('ascii'))
-        self.coord.start()
-
-        self._period = CONF.collect.period
-        self._wait_time = CONF.collect.wait_periods * self._period
+        self.coord.start(start_heart=True)
 
     def _lock(self, tenant_id):
         lock_name = b"cloudkitty-" + str(tenant_id).encode('ascii')
         return self.coord.get_lock(lock_name)
 
-    def _load_tenant_list(self):
-        self._tenants = self.fetcher.get_tenants()
-        random.shuffle(self._tenants)
-
     def _init_messaging(self):
         target = oslo_messaging.Target(topic='cloudkitty',
                                        server=CONF.host,
@@ -254,10 +275,10 @@ class Orchestrator(object):
         self.server.start()
 
     def _check_state(self, tenant_id):
-        timestamp = self.storage.get_state(tenant_id)
+        timestamp = self._state.get_state(tenant_id)
         return ck_utils.check_time_state(timestamp,
-                                         self._period,
-                                         self._wait_time)
+                                         CONF.collect.period,
+                                         CONF.collect.wait_periods)
 
     def process_messages(self):
         # TODO(sheeprine): Code kept to handle threading and asynchronous
@@ -268,26 +289,30 @@ class Orchestrator(object):
 
     def process(self):
         while True:
-            self.process_messages()
-            self._load_tenant_list()
-            while len(self._tenants):
-                for tenant in self._tenants[:]:
-                    lock = self._lock(tenant)
-                    if lock.acquire(blocking=False):
-                        if not self._check_state(tenant):
-                            self._tenants.remove(tenant)
-                        else:
-                            worker = Worker(self.collector,
-                                            self.storage,
-                                            tenant)
-                            worker.run()
-                        lock.release()
-                    self.coord.heartbeat()
+            self.tenants = self.fetcher.get_tenants()
+            random.shuffle(self.tenants)
+            LOG.info('Tenants loaded for fetcher %s', self.fetcher.name)
+
+            for tenant_id in self.tenants:
+
+                lock = self._lock(tenant_id)
+                if lock.acquire(blocking=False):
+                    state = self._check_state(tenant_id)
+                    if state:
+                        worker = Worker(
+                            self.collector,
+                            self.storage,
+                            tenant_id,
+                        )
+                        worker.run()
+
+                    lock.release()
+
                 # NOTE(sheeprine): Slow down looping if all tenants are
                 # being processed
                 eventlet.sleep(1)
             # FIXME(sheeprine): We may cause a drift here
-            eventlet.sleep(self._period)
+            eventlet.sleep(CONF.collect.period)
 
     def terminate(self):
         self.coord.stop()
