@@ -73,9 +73,19 @@ function is_cloudkitty_enabled {
     return 1
 }
 
+# Remove WSGI files, disable and remove Apache vhost file
+function _cloudkitty_cleanup_apache_wsgi {
+    if is_service_enabled ck-api && [ "$CLOUDKITTY_USE_MOD_WSGI" == "True" ]; then
+        sudo rm -f "$CLOUDKITTY_WSGI_DIR"/*
+        sudo rm -rf  "$CLOUDKITTY_WSGI_DIR"
+        sudo rm -f $(apache_site_config_for cloudkitty)
+    fi
+}
+
 # cleanup_cloudkitty() - Remove residual data files, anything left over from previous
 # runs that a clean run would need to clean up
 function cleanup_cloudkitty {
+    _cloudkitty_cleanup_apache_wsgi
     # Clean up dirs
     rm -rf $CLOUDKITTY_AUTH_CACHE_DIR/*
     rm -rf $CLOUDKITTY_CONF_DIR/*
@@ -85,6 +95,31 @@ function cleanup_cloudkitty {
     done
 }
 
+
+# Configure mod_wsgi
+function _cloudkitty_config_apache_wsgi {
+    sudo mkdir -m 755 -p $CLOUDKITTY_WSGI_DIR
+
+    local cloudkitty_apache_conf=$(apache_site_config_for cloudkitty)
+    local venv_path=""
+
+    # Copy proxy vhost and wsgi file
+    sudo cp $CLOUDKITTY_DIR/cloudkitty/api/app.wsgi $CLOUDKITTY_WSGI_DIR/app.wsgi
+
+    if [[ ${USE_VENV} = True ]]; then
+        venv_path="python-path=${PROJECT_VENV["cloudkitty"]}/lib/$(python_version)/site-packages"
+    fi
+
+    sudo cp $CLOUDKITTY_DIR/devstack/apache-cloudkitty.template $cloudkitty_apache_conf
+    sudo sed -e "
+        s|%PORT%|$CLOUDKITTY_SERVICE_PORT|g;
+        s|%APACHE_NAME%|$APACHE_NAME|g;
+        s|%WSGIAPP%|$CLOUDKITTY_WSGI_DIR/app.wsgi|g;
+        s|%USER%|$STACK_USER|g;
+        s|%VIRTUALENV%|$venv_path|g
+    " -i $cloudkitty_apache_conf
+}
+
 # configure_cloudkitty() - Set config files, create data dirs, etc
 function configure_cloudkitty {
     setup_develop $CLOUDKITTY_DIR
@@ -97,18 +132,24 @@ function configure_cloudkitty {
 
     touch $CLOUDKITTY_CONF
 
-    cp $CLOUDKITTY_DIR$CLOUDKITTY_CONF_DIR/policy.json $CLOUDKITTY_CONF_DIR
+    # generate policy sample file
+    oslopolicy-sample-generator --config-file $CLOUDKITTY_DIR/etc/oslo-policy-generator/cloudkitty.conf --output-file $CLOUDKITTY_DIR/etc/cloudkitty/policy.yaml.sample
+    cp $CLOUDKITTY_DIR/etc/cloudkitty/policy.yaml.sample "$CLOUDKITTY_CONF_DIR/policy.yaml"
+    iniset $CLOUDKITTY_CONF oslo_policy policy_file 'policy.yaml'
+
     cp $CLOUDKITTY_DIR$CLOUDKITTY_CONF_DIR/api_paste.ini $CLOUDKITTY_CONF_DIR
+    cp $CLOUDKITTY_DIR$CLOUDKITTY_CONF_DIR/metrics.yml $CLOUDKITTY_CONF_DIR
     iniset_rpc_backend cloudkitty $CLOUDKITTY_CONF DEFAULT
 
     iniset $CLOUDKITTY_CONF DEFAULT notification_topics 'notifications'
     iniset $CLOUDKITTY_CONF DEFAULT debug "$ENABLE_DEBUG_LOG_LEVEL"
+    iniset $CLOUDKITTY_CONF DEFAULT auth_strategy $CLOUDKITTY_AUTH_STRATEGY
 
     # auth
     iniset $CLOUDKITTY_CONF authinfos auth_type v3password
     iniset $CLOUDKITTY_CONF authinfos auth_protocol http
-    iniset $CLOUDKITTY_CONF authinfos auth_url "$KEYSTONE_SERVICE_PROTOCOL://$KEYSTONE_SERVICE_HOST:5000/v3"
-    iniset $CLOUDKITTY_CONF authinfos identity_uri "$KEYSTONE_SERVICE_PROTOCOL://$KEYSTONE_SERVICE_HOST:5000/v3"
+    iniset $CLOUDKITTY_CONF authinfos auth_url "$KEYSTONE_SERVICE_URI/v3"
+    iniset $CLOUDKITTY_CONF authinfos identity_uri "$KEYSTONE_SERVICE_URI/v3"
     iniset $CLOUDKITTY_CONF authinfos username cloudkitty
     iniset $CLOUDKITTY_CONF authinfos password $SERVICE_PASSWORD
     iniset $CLOUDKITTY_CONF authinfos project_name $SERVICE_TENANT_NAME
@@ -118,25 +159,51 @@ function configure_cloudkitty {
     iniset $CLOUDKITTY_CONF authinfos project_domain_name default
     iniset $CLOUDKITTY_CONF authinfos debug "$ENABLE_DEBUG_LOG_LEVEL"
 
-    iniset $CLOUDKITTY_CONF keystone_fetcher auth_section authinfos
-    iniset $CLOUDKITTY_CONF keystone_fetcher keystone_version 3
+    iniset $CLOUDKITTY_CONF fetcher backend $CLOUDKITTY_FETCHER
+    iniset $CLOUDKITTY_CONF "fetcher_$CLOUDKITTY_FETCHER" auth_section authinfos
+    if [[ "$CLOUDKITTY_FETCHER" == "keystone" ]]; then
+        iniset $CLOUDKITTY_CONF fetcher_keystone keystone_version 3
+    fi
+
+    if [ "$CLOUDKITTY_STORAGE_BACKEND" == "influxdb" ]; then
+        iniset $CLOUDKITTY_CONF storage_${CLOUDKITTY_STORAGE_BACKEND} user ${CLOUDKITTY_INFLUXDB_USER}
+        iniset $CLOUDKITTY_CONF storage_${CLOUDKITTY_STORAGE_BACKEND} password ${CLOUDKITTY_INFLUXDB_PASSWORD}
+        iniset $CLOUDKITTY_CONF storage_${CLOUDKITTY_STORAGE_BACKEND} database ${CLOUDKITTY_INFLUXDB_DATABASE}
+        iniset $CLOUDKITTY_CONF storage_${CLOUDKITTY_STORAGE_BACKEND} host ${CLOUDKITTY_INFLUXDB_HOST}
+        iniset $CLOUDKITTY_CONF storage_${CLOUDKITTY_STORAGE_BACKEND} port ${CLOUDKITTY_INFLUXDB_PORT}
+    fi
 
     # collect
     iniset $CLOUDKITTY_CONF collect collector $CLOUDKITTY_COLLECTOR
-    iniset $CLOUDKITTY_CONF ${CLOUDKITTY_COLLECTOR}_collector auth_section authinfos
-    iniset $CLOUDKITTY_CONF collect services $CLOUDKITTY_SERVICES
+    iniset $CLOUDKITTY_CONF "collector_${CLOUDKITTY_COLLECTOR}" auth_section authinfos
+    iniset $CLOUDKITTY_CONF collect metrics_conf $CLOUDKITTY_CONF_DIR/$CLOUDKITTY_METRICS_CONF
 
     # output
     iniset $CLOUDKITTY_CONF output backend $CLOUDKITTY_OUTPUT_BACKEND
     iniset $CLOUDKITTY_CONF output basepath $CLOUDKITTY_OUTPUT_BASEPATH
     iniset $CLOUDKITTY_CONF output pipeline $CLOUDKITTY_OUTPUT_PIPELINE
 
+    # storage
+    iniset $CLOUDKITTY_CONF storage backend $CLOUDKITTY_STORAGE_BACKEND
+    iniset $CLOUDKITTY_CONF storage version $CLOUDKITTY_STORAGE_VERSION
+
     # database
     local dburl=`database_connection_url cloudkitty`
     iniset $CLOUDKITTY_CONF database connection $dburl
 
     # keystone middleware
     configure_auth_token_middleware $CLOUDKITTY_CONF cloudkitty $CLOUDKITTY_AUTH_CACHE_DIR
+
+    if is_service_enabled ck-api && [ "$CLOUDKITTY_USE_MOD_WSGI" == "True" ]; then
+        _cloudkitty_config_apache_wsgi
+    fi
+}
+
+function wait_for_gnocchi() {
+    local gnocchi_url=$(openstack --os-cloud devstack-admin endpoint list --service metric --interface public -c URL -f value)
+    if ! wait_for_service $SERVICE_TIMEOUT $gnocchi_url; then
+       die $LINENO "Waited for gnocchi too long."
+    fi
 }
 
 # create_cloudkitty_cache_dir() - Part of the init_cloudkitty() process
@@ -161,6 +228,12 @@ function create_cloudkitty_data_dir {
     sudo chown $STACK_USER $CLOUDKITTY_DATA_DIR/locks
 }
 
+function create_influxdb_database {
+    if [ "$CLOUDKITTY_STORAGE_BACKEND" == "influxdb" ]; then
+        influx -execute "CREATE DATABASE ${CLOUDKITTY_INFLUXDB_DATABASE}"
+    fi
+}
+
 # init_cloudkitty() - Initialize CloudKitty database
 function init_cloudkitty {
     # Delete existing cache
@@ -176,28 +249,66 @@ function init_cloudkitty {
     # (Re)create cloudkitty database
     recreate_database cloudkitty utf8
 
+    create_influxdb_database
+
     # Migrate cloudkitty database
     $CLOUDKITTY_BIN_DIR/cloudkitty-dbsync upgrade
 
     # Init the storage backend
+    if [ $CLOUDKITTY_STORAGE_BACKEND == 'hybrid' ]; then
+        wait_for_gnocchi
+    fi
     $CLOUDKITTY_BIN_DIR/cloudkitty-storage-init
 
     create_cloudkitty_cache_dir
     create_cloudkitty_data_dir
 }
 
+function install_influx_ubuntu {
+    local influxdb_file=$(get_extra_file https://dl.influxdata.com/influxdb/releases/influxdb_1.6.3_amd64.deb)
+    sudo dpkg -i --skip-same-version ${influxdb_file}
+}
+
+function install_influx_fedora {
+    local influxdb_file=$(get_extra_file https://dl.influxdata.com/influxdb/releases/influxdb-1.6.3.x86_64.rpm)
+    sudo yum localinstall -y ${influxdb_file}
+}
+
+function install_influx {
+    if is_ubuntu; then
+        install_influx_ubuntu
+    elif is_fedora; then
+        install_influx_fedora
+    else
+        die $LINENO "Distribution must be Debian or Fedora-based"
+    fi
+    sudo cp -f "${CLOUDKITTY_DIR}"/devstack/files/influxdb.conf /etc/influxdb/influxdb.conf
+    sudo systemctl start influxdb || sudo systemctl restart influxdb
+}
+
 # install_cloudkitty() - Collect source and prepare
 function install_cloudkitty {
     git_clone $CLOUDKITTY_REPO $CLOUDKITTY_DIR $CLOUDKITTY_BRANCH
     setup_develop $CLOUDKITTY_DIR
+
+    if [ $CLOUDKITTY_STORAGE_BACKEND == 'influxdb' ]; then
+        install_influx
+    fi
 }
 
 # start_cloudkitty() - Start running processes, including screen
 function start_cloudkitty {
-    screen_it ck-proc "cd $CLOUDKITTY_DIR; $CLOUDKITTY_BIN_DIR/cloudkitty-processor --config-file=$CLOUDKITTY_CONF"
-    screen_it ck-api "cd $CLOUDKITTY_DIR; $CLOUDKITTY_BIN_DIR/cloudkitty-api --config-file=$CLOUDKITTY_CONF"
+    run_process ck-proc "$CLOUDKITTY_BIN_DIR/cloudkitty-processor --config-file=$CLOUDKITTY_CONF"
+    if [[ "$CLOUDKITTY_USE_MOD_WSGI" == "False" ]]; then
+        run_process ck-api "$CLOUDKITTY_BIN_DIR/cloudkitty-api --config-file=$CLOUDKITTY_CONF"
+    elif is_service_enabled ck-api; then
+        enable_apache_site cloudkitty
+        echo_summary "Waiting 15s for cloudkitty-processor to authenticate against keystone before apache is restarted."
+        sleep 15s
+        restart_apache_server
+    fi
     echo "Waiting for ck-api ($CLOUDKITTY_SERVICE_HOST:$CLOUDKITTY_SERVICE_PORT) to start..."
-    if ! timeout $SERVICE_TIMEOUT sh -c "while ! wget --no-proxy -q -O- http://$CLOUDKITTY_SERVICE_HOST:$CLOUDKITTY_SERVICE_PORT; do sleep 1; done"; then
+    if ! wait_for_service $SERVICE_TIMEOUT $CLOUDKITTY_SERVICE_PROTOCOL://$CLOUDKITTY_SERVICE_HOST:$CLOUDKITTY_SERVICE_PORT; then
         die $LINENO "ck-api did not start"
     fi
 }
@@ -205,9 +316,19 @@ function start_cloudkitty {
 # stop_cloudkitty() - Stop running processes
 function stop_cloudkitty {
     # Kill the cloudkitty screen windows
-    for serv in ck-api ck-proc; do
-        screen_stop $serv
-    done
+    if is_service_enabled ck-proc ; then
+        stop_process ck-proc
+    fi
+
+    if is_service_enabled ck-api ; then
+        if [ "$CLOUDKITTY_USE_MOD_WSGI" == "True" ]; then
+            disable_apache_site cloudkitty
+            restart_apache_server
+        else
+            # Kill the cloudkitty screen windows
+            stop_process ck-api
+        fi
+    fi
 }
 
 # install_python_cloudkittyclient() - Collect source and prepare
